/*****************************************************************************************************/
/* Program:  frailty.sas                                                                             */
/* Purpose:  create flags for indicators of frailty and use indicators to calculate probability of   */
/*           ADL-Dependency based on Faurot, et al, PDS 2015                                         */
/* Location: This program and the accompanying frailtyEstimates.sas7bdat can be downloaded from the  */
/*           UNC Harry Guess website: http://sph.unc.edu/epid/harry-guess-research-community/.       */                             
/*                                                                                                   */
/* Overview: This macro requires three datasets as inputs: cohort dataset, diagnosis dataset and     */
/*           procedure dataset.  The cohort dataset must contain one record per cohort member and    */
/*           must contain a unique patient-level identifier and a cohort entry date.  The diagnosis  */
/*           dataset must contain one record per diagnosis and must contain the unique person-level  */
/*           identifier and the date of the diagnosis.  The procedure dataset must contain one       */
/*           record per HCPC procedure code and must contain the unique person-level identifier and  */
/*           the date of the procedure.                                                              */
/*                                                                                                   */
/* Output:   This macro will output a dataset with the same records as your input cohort dataset.    */
/*           The output dataset will contain your &IDVAR, &DATEVAR, a flag for each of the frailty   */
/*           parameters, and a variable named PREDICTED that contains the persons predicted          */
/*           probability of ADL-Dependence based on Faurot, et al, PDS 2015.                         */
/*                                                                                                   */
/* Macro parameters:                                                                                 */
/*    COHORTDS: name of cohort dataset containing one record per cohort member with the following    */
/*              demographic variables:                                                               */
/*              AGE:  numeric variable containing age at cohort entry date (used to calculate AGE65) */
/*              SEX:  numeric variable: 0=Male, 1=Female                                             */
/*              RACE: character variable: W=White, B=Black, H=Hispanic, O=Other                      */
/*                                                                                                   */
/*    IDVAR:    variable name of unique person-level identifier, present in all three datasets       */
/*    DATEVAR:  variable name of cohort entry date, present in &COHORTDS                             */
/*                                                                                                   */
/*    DXDS:     name of dataset containing one record per diagnosis                                  */
/*    DXVAR:    variable name of diagnosis code, present in &DXDS                                    */
/*    DXDATE:   variable name of date of diagnosis, present in &DXDS                                 */
/*                                                                                                   */
/*    PROCDS:   name of dataset containing one record per procedure                                  */
/*    PROCVAR:  variable name of procedure code, present in &PROCDS                                  */
/*    PROCDATE: variable name of date of procedure, present in &PROCDS                               */
/*                                                                                                   */
/*    MAXDAYS:  length of look-back period for assessing frailty                                     */
/*    LIB:      library name where FrailtyEstimates.sasbdat is stored                                */
/*    OUTDS:    name of output dataset (will contain                                                 */
/*****************************************************************************************************/

%macro frailty(cohortDS=out.cohort2009, idvar=baseid, datevar=svy_dt,
               dxds=allDX, dxvar=dgn, dxdate=from_dt,
               procds=allHCPC, procvar=hcpc, procdate=from_dt,
               maxDays=238, lib=frail, outds=frailty);

   proc sql;
      /* Create indicators based on diagnoses */
      create table _temp_dxflags_ as 
         select distinct a.&idvar, a.&datevar,

            case when substr(b.&dxvar,1,3) in ('710' '711' '712' '714' '715' '718' '725')
                   or substr(b.&dxvar,1,4) in ('7165' '7166' '7168' '7169' '7190' '7191' '7194' '7195' '7199') 
                then 1 else 0 end as arthritis,

            case when substr(b.&dxvar,1,4) in ('5965' '5996' '7882' '7883') 
                then 1 else 0 end as bladder,

            case when substr(b.&dxvar,1,3) in ('348' '430' '431' '432' '436' '852' '853' '854') 
                   or        b.&dxvar      in ('34982' '43301' '43311' '43321' '43331' '43391' '43401' '43411' '43491') 
                then 1 else 0 end as brain_inj,

            case when substr(b.&dxvar,1,3) in ('707')
                then 1 else 0 end as decub,

            case when substr(b.&dxvar,1,3) in ('290' '294' '331' '797')
                   or substr(b.&dxvar,1,4) in ('4380')
                   or        b.&dxvar      in ('33390' '33392' '33399' '78093')
                then 1 else 0 end as dement,

            case when substr(b.&dxvar,1,3) in ('425' '428')
                 or substr(b.&dxvar,1,4) in ('4290' '4291' '4293' '4294')
                then 1 else 0 end as hf,

            case when substr(b.&dxvar,1,3) in ('458')
                   or substr(b.&dxvar,1,4) in ('7855' '9584' '9980')
                then 1 else 0 end as hyposhock,

            case when substr(b.&dxvar,1,3) in ('272') 
                then 1 else 0 end as lipid,

            case when substr(b.&dxvar,1,3) in ('342' '344')
                   or substr(b.&dxvar,1,4) in ('4382' '4383' '4384' '4385' '7814') 
                then 1 else 0 end as paralysis,

            case when substr(b.&dxvar,1,3) in ('332') 
                then 1 else 0 end as pd,

            case when substr(b.&dxvar,1,3) in ('700' '703') 
                   or substr(b.&dxvar,1,4) in ('6811') 
                then 1 else 0 end as podiatric,

            case when substr(b.&dxvar,1,2) in ('29')
                   or substr(b.&dxvar,1,3) in ('310' '311')
                   or substr(b.&dxvar,1,4) in ('3000') 
                then 1 else 0 end as psych,

            case when substr(b.&dxvar,1,4) in ('V571' 'V573' 'V578' 'V579')
                   or        b.&dxvar      in ('V5721') 
                then 1 else 0 end as rehab, 

            case when substr(b.&dxvar,1,3) in ('V76')
                then 1 else 0 end as screening,

            case when substr(b.&dxvar,1,3) in ('386')
                   or substr(b.&dxvar,1,4) in ('7804')
                   or        b.&dxvar      in ('43885')
                then 1 else 0 end as vertigo,

            case when substr(b.&dxvar,1,4) in ('7282' '7283' '7993')
                   or        b.&dxvar      in ('72887' 'V4984') 
                then 1 else 0 end as weakness

         from &cohortds as a 
            left join &dxds as b
               on a.&idvar = b.&idvar and 0 <= a.&datevar-b.&dxdate %IF &maxDays ^=  %THEN <= &maxDays ;;


      /* Create indicators based on procedure */
      create table _temp_hcpcflags_ as 
         select distinct a.&idvar, a.&datevar,

            case when c.&procvar in ('A0426' 'A0427' 'A0428' 'A0429' 'A0999')
                then 1 else 0 end as ambulance,

            case when c.&procvar in ('E0250' 'E0251' 'E0255' 'E0256' 'E0260' 'E0261' 'E0265' 'E0266' 'E0270'
                                     'E0290' 'E0291' 'E0292' 'E0293' 'E0294' 'E0295' 'E0296' 'E0297' 'E0301'
                                     'E0302' 'E0303' 'E0304' 'E0316')
                then 1 else 0 end as hospbed,

            case when c.&procvar in ('E1390' 'E1391' 'E1392' 'E0431' 'E0433' 'E0434' 'E0435' 'E0439' 'E0441'
                                     'E0442' 'E0443')
                then 1 else 0 end as oxygen,

            case when c.&procvar in ('E1050' 'E1060' 'E1070' 'E1083' 'E1084' 'E1085' 'E1086' 'E1087' 'E1088' 
                                     'E1089' 'E1090' 'E1091' 'E1092' 'E1093' 'E1100' 'E1110' 'E1140' 'E1150'
                                     'E1160' 'E1161' 'E1170' 'K0001' 'K0002' 'K0003' 'K0004' 'K0005' 'K0006' 
                                     'K0007' 'K0008' 'K0009')
                then 1 else 0 end as wheelchair

         from &cohortds as a 
            left join &procds as c
               on a.&idvar = c.&idvar and 0 <= a.&datevar-c.&procdate %IF &maxDays ^=  %THEN <= &maxDays ;;


      /* Combine diagnoses and procedures and collapse into person level dataset */
      create table _temp_cohort_ as
         select distinct a.&idvar, a.&datevar, a.sex, a.race,
            a.age - 65 as age65, calculated age65 * calculated age65 as age65sq,  
            case when a.race='B' then 1 else 0 end as raceB,
            case when a.race='H' then 1 else 0 end as raceH,
            case when a.race='O' then 1 else 0 end as raceO,
            max(b.arthritis) as arthritis label='Arthritis',
            max(b.bladder) as bladder label='Bladder dysfunction',
            max(b.brain_inj) as brain_inj label='Stroke/brain injury',
            max(b.decub) as decub label='Skin ulcer (decubitus)',
            max(b.dement) as dement label='Dementias',
            max(b.hf) as hf label='Heart failure',
            max(b.hyposhock) as hyposhock label='Hypotension or shock',
            max(b.lipid) as lipid label='Lipid abnormality',
            max(b.paralysis) as paralysis label='Paralysis',
            max(b.pd) as pd label='Parkinsons disease',
            max(b.podiatric) as podiatric label='Podiatric care',
            max(b.psych) as psych label='Psychiatric illness',
            max(b.rehab) as rehab label='Rehabilitation care', 
            max(b.screening) as screening label='Cancer screening',
            max(b.vertigo) as vertigo label='Vertigo',
            max(b.weakness) as weakness label='Weakness',
            max(c.ambulance) as ambulance label='Ambulance',
            max(c.hospbed) as hospbed label='Home Hospital Bed',
            max(c.oxygen) as oxygen label='Home oxygen',
            max(c.wheelchair) as wheelchair label='Wheelchair'
         from &cohortds as a
            left join _temp_dxflags_ as b on a.&idvar = b.&idvar and a.&datevar = b.&datevar
            left join _temp_hcpcflags_ as c on a.&idvar = c.&idvar and a.&datevar = c.&datevar
         group by a.&idvar, a.&datevar;
   quit;

   /* Apply estimates from Faurot, et al */
   %IF %SYSFUNC(EXIST(&lib..frailtyEstimates)) %THEN %DO;
      data &outds; 
         if _N_ = 1 then set &lib..frailtyEstimates;
         set _temp_cohort_; 

         odds = est_intercept + est_age65*age65 + est_age65sq*age65sq + est_sex*sex +
                est_raceB*raceB + est_raceH*raceH + est_raceO*raceO + 
                est_screening*screening + est_lipid*lipid + est_vertigo*vertigo + 
                est_arthritis*arthritis + est_bladder*bladder + est_podiatric*podiatric + 
                est_hf*hf + est_psych*psych + est_rehab*rehab + est_oxygen*oxygen +
                est_hyposhock*hyposhock + est_ambulance*ambulance +  est_brain_inj*brain_inj + 
                est_dement*dement + est_pd*pd + est_weakness*weakness + est_decub*decub +
                est_paralysis*paralysis + est_wheelchair*wheelchair + est_hospbed*hospbed;
         predicted = exp(odds) / (1+exp(odds));
         drop age65sq raceB raceH raceO odds;
      run;
   %END; %ELSE %DO;
      data &outds;
         set _temp_cohort_;
         odds = -3.7466 + 0.0028*age65 + 0.0021*age65*age65 + 0.3530*sex + 
                0.3126*(race='B') + -0.5097*(race='H') + 0.8228*(race='O') +
                -0.5559*screening + -0.3544*lipid + -0.5925*vertigo + 
                0.3598*arthritis + 0.4161*bladder + 0.5322*podiatric + 
                0.4120*hf + 0.5363*psych + -0.4315*rehab + 0.9094*oxygen + 
                0.6038*hyposhock + 0.4423*ambulance + 0.4745*brain_inj + 
                0.7917*dement + 1.1873*pd + 0.4921*weakness + 0.8464*decub + 
                1.8541*paralysis +1.5094*wheelchair + 1.4995*hospbed;
         predicted = exp(odds) / (1+exp(odds));
         drop age65sq raceB raceH raceO odds;
      run;
   %END;

   /* Clean up temporary datasets */
   proc datasets lib=work nolist nodetails; delete _temp_dxflags_ _temp_hcpcflags_ _temp_cohort_; run;quit;
%mend frailty;

